//  MClock - Yet Another Clock program
//
//  Demonstrates the timer, a dialog box (Help/About), font sizing,
//  and window sizing.
//
//  Mario Giannini

#include <windows.h>
#include <time.h>
#include <stdio.h>
#include "resource.h"

// Function Prototypes (not all needed, but here anyway)
BOOL CALLBACK AboutDlgProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );
LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM );
void SizeForClock( HWND hWnd );
int WINAPI WinMain( HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpszArgs, int nCmdShow );
void ScaleFont( int Width, int Height );
void GetTimeStr( HWND hWnd, char * Dest, time_t* Last );
void ToggleMenuItemCheck( HWND hWnd, UINT ID );
LRESULT HandleMenu( HWND hWnd, WPARAM wParam, LPARAM lParam );

// Global Variables
char szAppName[]= "MClock";
HINSTANCE hInstance;
HFONT MyFont;

// SizeForClock - Sets Initial main-window size to match time display
void SizeForClock( HWND hWnd )
{
	RECT Client, Window;
	HDC hDC = GetDC( hWnd );
	SIZE Size;
	GetWindowRect( hWnd, &Window );
	GetClientRect( hWnd, &Client );
	GetTextExtentPoint32( hDC, "12:22:22 PM", 11, &Size );
	Size.cx *= 2; // *2 because the menu is actually longer than our time text.
	ReleaseDC( hWnd, hDC );
	Size.cy += Window.bottom - Window.top - Client.bottom;
	Size.cx += Window.right - Window.left - Client.right;
	SetWindowPos( hWnd, NULL, 0, 0, Size.cx, Size.cy, SWP_NOMOVE|SWP_NOZORDER );
} 

int WINAPI WinMain( HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpszArgs, int nCmdShow )
{
	HWND hWnd;
	MSG Msg;
	WNDCLASS WndClass;

	hInstance = hInst;

	WndClass.hInstance = hInst;
	WndClass.lpszClassName = szAppName;
	WndClass.lpfnWndProc = WndProc;
	WndClass.style = 0;	// No special styles

	// Set resources for the program: Icon, Cursor, Menu, and background
	WndClass.hIcon = LoadIcon( hInst, MAKEINTRESOURCE(IDI_ICON) );
	WndClass.hCursor = LoadCursor( NULL, IDC_ARROW );
	WndClass.lpszMenuName = MAKEINTRESOURCE(IDR_MENU1);
	WndClass.hbrBackground = (HBRUSH) GetStockObject( WHITE_BRUSH );

	WndClass.cbClsExtra = 0; // 'Extra' information about this window,
	WndClass.cbWndExtra = 0; // our program has none.

	// Register the class (this includes the 'WndProc')
	if( !RegisterClass( &WndClass ) )
	{
		MessageBox( NULL, "Error registering class", NULL, MB_ICONEXCLAMATION|MB_OK );
		return( 0 );
	}

	hWnd = CreateWindow( szAppName, szAppName, WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, 
		HWND_DESKTOP, NULL, hInst, NULL );

	if( !hWnd )
	{
		MessageBox( NULL, "Error creating window", NULL, MB_ICONEXCLAMATION|MB_OK );
		return( 0 );
	}

	SizeForClock( hWnd ); // Size the parent window to fit tightly around the displayed digits

	ShowWindow( hWnd, nCmdShow );
	UpdateWindow( hWnd );

	while( GetMessage( &Msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &Msg );
		DispatchMessage( &Msg );
	}
	
	return( Msg.wParam );
}

// ScaleFont - Creates font sized to fill the parent window
void ScaleFont( int Width, int Height )
{
	if( MyFont )
		DeleteObject( MyFont );
	MyFont=CreateFont( Height, Width/13, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, 
	DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
		DEFAULT_QUALITY, DEFAULT_PITCH|FF_DONTCARE,"Arial"); 
	// The /13 was determined because the Width parameter to create
	// font is the *average* character width.  Since we have upto 12
	// characters (12:00:00 AM), 1 was added to accomodate the fact
	// that it's a proportionate font we're working with.
}

// GetTimeStr - Formats the time in Military or AMPM, according to menu status
void GetTimeStr( HWND hWnd, char * Dest, time_t* Last )
{
	struct tm* TheTime;
	int Hour;
	
	time( Last );
	TheTime = localtime( Last );
	if( GetMenuState( GetMenu(hWnd), IDM_VIEW_AMPM, MF_BYCOMMAND ) == MF_CHECKED )
	{
		Hour = TheTime->tm_hour%12;
		Hour = Hour ? Hour : 12;
		sprintf( Dest, "%02d:%02d:%02d %cM", Hour, 
			TheTime->tm_min, TheTime->tm_sec, TheTime->tm_hour >=12?'P':'A' );
		
	}
	else
		sprintf( Dest, "%02d:%02d:%02d", TheTime->tm_hour, 
			TheTime->tm_min, TheTime->tm_sec );
}

// ToggleMenuItemCheck - Toggles check status of a menu item
void ToggleMenuItemCheck( HWND hWnd, UINT ID )
{
	HMENU hMenu = GetMenu(hWnd);
	if( CheckMenuItem( hMenu, ID, MF_BYCOMMAND | MF_CHECKED ) == MF_CHECKED )
		CheckMenuItem( hMenu, ID, MF_BYCOMMAND | MF_UNCHECKED );
}

// HandleMenu - Handles all menu choices from main menu
LRESULT HandleMenu( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	switch( LOWORD( wParam ) )
	{
		case IDM_FILE_EXIT:
			DestroyWindow( hWnd );
			break;
		case IDM_VIEW_AMPM:
			ToggleMenuItemCheck( hWnd, IDM_VIEW_AMPM );
			InvalidateRect( hWnd, NULL, TRUE );
			break;
		case IDM_HELP_ABOUT: // Displayes the Help/About dialog
			DialogBox( hInstance, MAKEINTRESOURCE( IDD_ABOUT),
				hWnd, (DLGPROC)AboutDlgProc );
			return(0);
	}
	return( 0 );

}

// AboutDlgProc - Callback procedure for About Dialog box
BOOL CALLBACK AboutDlgProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch( uMsg )
	{
		case WM_INITDIALOG:
			return( 1 );  // Return 1 if we don't set the focus to a control
		case WM_COMMAND:
			if( LOWORD( wParam ) == IDOK )
			{
				EndDialog( hwndDlg, 0 ); // This is how you close a Dialog box
				return(1);
			}
	}
	return( 0 ); // Dialog procedures return 0 if the message is not handled
	// Dialog procedures do *not* call DefWindowProc
}

LRESULT CALLBACK WndProc( HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam )
{
	char TimeStr[16];
	static time_t Last;
	time_t Now;
	HFONT OldFont;
	RECT Rect;

	switch( Message )
	{
		case WM_SIZE: // User resized window, scale font to fill it's new size
			ScaleFont( LOWORD( lParam ), HIWORD( lParam ) );
			if( wParam==SIZE_MAXIMIZED || wParam==SIZE_RESTORED )
				SetWindowText( hWnd, szAppName);
			InvalidateRect( hWnd, NULL, TRUE );
			break;

		case WM_COMMAND:
			return(HandleMenu( hWnd, wParam, lParam ) );

		case WM_CREATE: // Start the timer
			SetTimer( hWnd, 999, 100, NULL );
			break;

		case WM_PAINT: // Display the current time
			HDC DC;
			PAINTSTRUCT PaintStruct;
			GetTimeStr( hWnd, TimeStr, &Last );
			DC = BeginPaint( hWnd, &PaintStruct );
			GetClientRect( hWnd, &Rect );
			OldFont = SelectObject( DC, MyFont );
			DrawText( DC, TimeStr, strlen(TimeStr), &Rect, DT_CENTER );
			SelectObject( DC, OldFont );
			EndPaint( hWnd, &PaintStruct );
			break;

		case WM_TIMER:  // Check to see if we need to update time display
			time( &Now );
			if( Now != Last )
			{
				if( IsIconic( hWnd ) )
				{
					// If window is minimized, chhange window text
					GetTimeStr( hWnd, TimeStr, &Last );
					SetWindowText( hWnd, TimeStr );
				}
				else
					InvalidateRect( hWnd, NULL, FALSE ); // else force a WM_PAINT
			}
			break;

		case WM_DESTROY:
			KillTimer( hWnd, 999 );
			DeleteObject( MyFont );
			PostQuitMessage(0);
			break;

		default:
			return( DefWindowProc( hWnd, Message, wParam, lParam ) );
	}
	return( 0 );
}
